--- old/usr/src/lib/libzfs/common/libzfs_import.c	2015-08-27 12:50:06.787121263 -0400
+++ new/usr/src/lib/libzfs/common/libzfs_import.c	2015-08-27 12:50:06.793455561 -0400
@@ -240,6 +240,7 @@
 		ne->ne_guid = vdev_guid;
 		ne->ne_next = pl->names;
 		pl->names = ne;
+		nvlist_free(config);
 		return (0);
 	}
 
@@ -273,10 +274,8 @@
 	}
 
 	if (pe == NULL) {
-		if ((pe = zfs_alloc(hdl, sizeof (pool_entry_t))) == NULL) {
-			nvlist_free(config);
+		if ((pe = zfs_alloc(hdl, sizeof (pool_entry_t))) == NULL)
 			return (-1);
-		}
 		pe->pe_guid = pool_guid;
 		pe->pe_next = pl->pools;
 		pl->pools = pe;
@@ -292,19 +291,33 @@
 	}
 
 	if (ve == NULL) {
-		if ((ve = zfs_alloc(hdl, sizeof (vdev_entry_t))) == NULL) {
-			nvlist_free(config);
+		if ((ve = zfs_alloc(hdl, sizeof (vdev_entry_t))) == NULL)
 			return (-1);
-		}
 		ve->ve_guid = top_guid;
 		ve->ve_next = pe->pe_vdevs;
 		pe->pe_vdevs = ve;
 	}
 
 	/*
-	 * Third, see if we have a config with a matching transaction group.  If
-	 * so, then we do nothing.  Otherwise, add it to the list of known
-	 * configs.
+	 * Third, add the vdev guid -> path mappings so that we can fix up
+	 * the configuration as necessary before doing the import.
+	 */
+	if ((ne = zfs_alloc(hdl, sizeof (name_entry_t))) == NULL)
+		return (-1);
+
+	if ((ne->ne_name = zfs_strdup(hdl, path)) == NULL) {
+		free(ne);
+		return (-1);
+	}
+
+	ne->ne_guid = vdev_guid;
+	ne->ne_next = pl->names;
+	pl->names = ne;
+
+	/*
+	 * Finally, see if we have a config with a matching transaction
+	 * group.  If so, then we do nothing.  Otherwise, add it to the list
+	 * of known configs.
 	 */
 	for (ce = ve->ve_configs; ce != NULL; ce = ce->ce_next) {
 		if (ce->ce_txg == txg)
@@ -312,10 +325,8 @@
 	}
 
 	if (ce == NULL) {
-		if ((ce = zfs_alloc(hdl, sizeof (config_entry_t))) == NULL) {
-			nvlist_free(config);
+		if ((ce = zfs_alloc(hdl, sizeof (config_entry_t))) == NULL)
 			return (-1);
-		}
 		ce->ce_txg = txg;
 		ce->ce_config = config;
 		ce->ce_next = ve->ve_configs;
@@ -324,24 +335,6 @@
 		nvlist_free(config);
 	}
 
-	/*
-	 * At this point we've successfully added our config to the list of
-	 * known configs.  The last thing to do is add the vdev guid -> path
-	 * mappings so that we can fix up the configuration as necessary before
-	 * doing the import.
-	 */
-	if ((ne = zfs_alloc(hdl, sizeof (name_entry_t))) == NULL)
-		return (-1);
-
-	if ((ne->ne_name = zfs_strdup(hdl, path)) == NULL) {
-		free(ne);
-		return (-1);
-	}
-
-	ne->ne_guid = vdev_guid;
-	ne->ne_next = pl->names;
-	pl->names = ne;
-
 	return (0);
 }
 
@@ -1258,8 +1251,10 @@
 					(void) strlcpy(end, slice->rn_name,
 					    pathleft);
 					if (add_config(hdl, &pools, path,
-					    config) != 0)
+					    config) != 0) {
+						nvlist_free(config);
 						config_failed = B_TRUE;
+					}
 				}
 			}
 			free(slice->rn_name);
